﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace XmlSerializersTest
{
    class SampleAttributes
    {
        public string Title { get; private set; }
        public Dictionary<string, string> SerializersToSkip { get; private set; }

        public SampleAttributes(object sample)
        {
            SetDefaults(sample);
            var all = GetAttributes(sample);

            Process<TitleAttribute>(all, attr => { Title = attr.Title; });
            Process<SkipSerializerAttribute>(all, attr => { SerializersToSkip[attr.Serializer] = attr.Reason; });
        }

        private void SetDefaults(object sample)
        {
            Title = sample.GetType().Name;
            SerializersToSkip = new Dictionary<string, string>();
        }

        private Dictionary<Type, List<object>> GetAttributes(object sample)
        {
            var attributes = new Dictionary<Type, List<object>>();

            foreach (var attribute in sample.GetType().GetCustomAttributes(true))
            {
                List<object> list = null;
                if (!attributes.TryGetValue(attribute.GetType(), out list))
                {
                    list = new List<object>();
                    attributes[attribute.GetType()] = list;
                }
                list.Add(attribute);
            }

            return attributes;
        }

        private void Process<T>(Dictionary<Type, List<object>> attributes, Action<T> action)
        {
            List<object> list = null;
            if (!attributes.TryGetValue(typeof(T), out list)) return;

            foreach (T item in list)
            {
                action(item);
            }
        }
    }
}
